#!/usr/bin/perl

use warnings;
use strict;

use POSIX;
use Fuse;
use IO::Dir;
use Data::Dumper;

my $filesystem = {
	root => {
		content => {}
	}
};

my $lastAttredFile;

sub updateProcs {
	# FS leeren
	$filesystem = {	root => { content => {}	} };

	my $procdir = IO::Dir->new( "/proc" );
	if ( !defined( $procdir ) ) {
		die ( "Unable to open proc dir" );
	}

	# Alle Prozess-IDs
	while ( defined( $_ = $procdir->read ) ) {
		next if ( /^\D/ );
		my $PID = $_;
		my $uid = 0;
		my $name = 'na';

		open ( my $fhstatus, "/proc/$PID/status" );
		while ( <$fhstatus> ) {
			chomp;
			if ( m/^Uid:\s+(\d+)/ ) {
				$uid = $1;
			}
			if ( m/^Name:\s+(.+)$/ ) {
				$name = $1;
			}
		}
		close( $fhstatus );

		# Slashes, etc. ersetzen
		$name =~ s/[\:\-\@\/.\,*\]\[\s\)\(]/_/g;

		# UID-Verzeichnis im FS-Tree erstellen, falls es noch nicht
		# existiert
		$filesystem->{root}->{content}->{$uid} = { type => 'dir', content => {} } if
			( !defined( $filesystem->{root}->{content}->{$uid} ) );

		# PID und Name als Datei im FS-Tree speichern
		$filesystem->{root}->{content}->{$uid}->{content}->{$PID.'_'.$name} = { type => 'file' };
	}
}

sub my_getattr {
	my ( $filename ) = @_;
	print "Requesting Attribs [$filename]: ";

	# regulaere Datei
	my $type = 0100;

	# owner: r/w, group: r, others: r
	my $bits = 0644;

	# falls Verzeichnis, type auf dir setzen,
	# mode auf 0755:
	# owner: r/w/x, group: r/x, others: r/x
	my $current = $filesystem->{'root'}->{'content'};
	my @pathElements = split( '/', $filename );
	my $currentType = 'file';

	if ( @pathElements > 1 ) {
		foreach my $pathElement ( @pathElements[1..$#pathElements] ) {
			if ( !defined( $current->{$pathElement} ) ) {
				return -1*ENOENT;
			}

			$currentType = $current->{$pathElement}->{'type'};
			$lastAttredFile = $current->{$pathElement} if ( $currentType eq 'file' );

			$current = $current->{$pathElement}->{'content'} if ( $currentType eq 'dir' );
		}
	}

	if ( $filename eq '/' || $currentType eq 'dir' ) {
		$type = 0040;
		$bits = 0755;
		print "DIR\n"
	} else {
		print "FILE\n";
	}

	#my $mode = $type << 9 | $bits;
	my $mode = $type << 9 | $bits;
	my $nlink = 1;

	# reale UID (siehe perlvar)
	my $uid = $<;

	# reale GID (siehe perlvar)
	my ($gid) = split / /, $(;

	# Geraete-ID (special files only)
	my $rdev = 0;

	# letzter Zugriff
	my $atime = time;

	# Groeße
	my $size = 0;

	# letzte Aenderung
	my $mtime = $atime;

	# letzte Aenderung Inode
	my $ctime = $atime;

	my $blksize = 1024;
	my $blocks = 1;
	
	my $dev = 0;
	my $ino = 0;

	return ( $dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize,	$blocks	);
}

sub my_getdir {
	my ( $filename ) = @_;
	print "updating procfs...";
	updateProcs;
	print "OK\n";
	print "getdir requesting [$filename]\n";

	my $current = $filesystem->{'root'}->{'content'};
	my @pathElements = split( '/', $filename );

	if ( @pathElements > 1 ) {
		foreach my $pathElement ( @pathElements[1..$#pathElements] ) {
			return( -1*ENOENT ) if ( !defined( $current->{$pathElement} ) );
			$current = $current->{$pathElement}->{'content'};
		}
	}

	return( '.', keys( %{$current} ), 0 );
}

sub my_unlink {
	my ( $filename ) = @_;
	my( $pid ) = ($filename =~ m/\/(\d+)_.+$/ );

	print "Killing: [$pid]\n";
	kill( 9, $pid );

	return 0;
}

updateProcs;

Fuse::main(
	mountpoint	=> '/home/sxw/tmp/mnt',
	getdir		=> \&my_getdir,
	getattr		=> \&my_getattr,
	unlink		=> \&my_unlink,
	debug => 0
);



