#!/usr/bin/perl

use warnings;
use strict;

use POSIX;
use Fuse;
use Net::SMTP;

my $filesystem = {
	root => {
		content => {
			'simon@ruhr.pm.org' => { type => 'dir' },
			'simonaw@openoffice.org' => { type => 'dir' },
			'christoph.schlingensief@googlemail.com' => { type => 'dir' }
		}
	}
};


my $lastAttredFile;

my $currentBody;
my $currentMailAddress;
my $currentSubject;

sub my_getattr {
	my ( $filename ) = @_;
	print "Requesting Attribs [$filename]: ";

	# regulaere Datei
	my $type = 0100;

	# owner: r/w, group: r, others: r
	my $bits = 0644;

	# falls Verzeichnis, type auf dir setzen,
	# mode auf 0755:
	# owner: r/w/x, group: r/x, others: r/x
	my $current = $filesystem->{'root'}->{'content'};
	my @pathElements = split( '/', $filename );
	my $currentType = 'file';

	if ( @pathElements > 1 ) {
		foreach my $pathElement ( @pathElements[1..$#pathElements] ) {
			#print 'Checking for: ', $current->{$pathElement}, "\n";
			if ( !defined( $current->{$pathElement} ) ) {
				return -1*ENOENT;
			}
			$currentType = $current->{$pathElement}->{'type'};
			$lastAttredFile = $current->{$pathElement} if ( $currentType eq 'file' );

			$current = $current->{$pathElement}->{'content'} if ( $currentType eq 'dir' );
		}
	}

	if ( $filename eq '/' || $currentType eq 'dir' ) {
		$type = 0040;
		$bits = 0755;
		print "DIR\n"
	} else {
		print "FILE\n";
	}

	#my $mode = $type << 9 | $bits;
	my $mode = $type << 9 | $bits;
	my $nlink = 1;

	# reale UID (siehe perlvar)
	my $uid = $<;

	# reale GID (siehe perlvar)
	my ($gid) = split / /, $(;

	# Geraete-ID (special files only)
	my $rdev = 0;

	# letzter Zugriff
	my $atime = time;

	# Groeße
	my $size = 0;

	# letzte Aenderung
	my $mtime = $atime;

	# letzte Aenderung Inode
	my $ctime = $atime;

	my $blksize = 1024;
	my $blocks = 1;
	
	my $dev = 0;
	my $ino = 0;

	return ( $dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize,	$blocks	);
}

sub my_getdir {
	my ( $filename ) = @_;
	print "getdir requesting [$filename]\n";

	my $current = $filesystem->{'root'}->{'content'};
	my @pathElements = split( '/', $filename );

	if ( @pathElements > 1 ) {
		foreach my $pathElement ( @pathElements[1..$#pathElements] ) {
			return( -1*ENOENT ) if ( !defined( $current->{$pathElement} ) );
			$current = $current->{$pathElement}->{'content'};
		}
	}

	return( '.', keys( %{$current} ), 0 );
}

sub my_write {
	my ( $filename, $buffer, $offset ) = @_;

	print( "Writing [$filename] Offset: $offset\n" );
	print "Buffer: [$buffer]\n";

	$currentBody .= $buffer;

	return length( $buffer );
}

sub my_mknod {
	my ( $filename, $mode, $dev ) = @_;
	print "MkNode $filename\n";

	my @pathElements = split( '/', $filename );
	$filesystem->{root}->{content}->{$pathElements[1]}->{content} = {
		$pathElements[2] => { type => 'file' }
	};

	$currentBody = '';
	$currentMailAddress = $pathElements[1];
	$currentSubject = $pathElements[2];

	return 0;
}

sub my_flush {
	my ( $filename ) = @_;
	print "Flushing: $filename\n";

	if ( $currentBody eq '' ) {
		return 0;
	}

	my $smtp = Net::SMTP->new( 'localhost' );
	$smtp->mail( 'simon@ruhr.pm.org' );
	$smtp->to( $currentMailAddress );
	$smtp->data();
	$smtp->datasend( "Subject: $currentSubject\n\n$currentBody" );
	$smtp->dataend();
	$smtp->quit();

	return 0;
}

Fuse::main(
	mountpoint	=> '/home/sxw/tmp/mnt',
	getdir		=> \&my_getdir,
	getattr		=> \&my_getattr,
	write		=> \&my_write,
	mknod		=> \&my_mknod,
	flush		=> \&my_flush,
	debug => 0
);

